@@ -1,8 +1,12 @@
+"""
+Realization of data operations.
+"""
+
+
 from abc import ABC
-from typing import Set, Dict, List, Union
+from typing import Iterable, Set, Dict, List, Optional, Union
 import numbers
 import re
-import typing
 
 import data_algebra
 import data_algebra.expr_parse
@@ -48,22 +52,26 @@ class ViewRepresentation(OperatorPlatform, ABC):
 
     column_names: List[str]
     column_set: data_algebra.OrderedSet.OrderedSet
-    column_types: typing.Optional[Dict[str, type]]
+    column_types: Optional[Dict[str, type]]
     sources: List[
         "ViewRepresentation"
     ]  # https://www.python.org/dev/peps/pep-0484/#forward-references
 
     def __init__(
         self,
-        column_names: List[str],
+        column_names: Iterable[str],
         *,
-        column_types: typing.Optional[Dict[str, type]] = None,
-        sources: List["ViewRepresentation"] = None,
+        column_types: Optional[Dict[str, type]] = None,
+        sources: Optional[List["ViewRepresentation"]] = None,
         node_name: str,
     ):
         if isinstance(column_names, str):
             column_names = [column_names]
-        self.column_names = [c for c in column_names]
+        else:
+            column_names = [v for v in column_names]  # make sure a list and a disjoint copy
+        assert isinstance(column_names, list)
+        assert all([isinstance(v, str) for v in column_names])
+        self.column_names = column_names
         for ci in self.column_names:
             assert isinstance(ci, str)
         if len(self.column_names) < 1:
@@ -624,15 +632,19 @@ class TableDescription(ViewRepresentation):
     def __init__(
         self,
         *,
-        table_name=None,
-        column_names,
+        table_name: Optional[str] = None,
+        column_names: Iterable[str],
         qualifiers=None,
         sql_meta=None,
-        column_types: typing.Optional[Dict[str, type]] = None,
+        column_types: Optional[Dict[str, type]] = None,
         head=None,
-        limit_was=None,
-        nrows=None,
+        limit_was: Optional[int] = None,
+        nrows: Optional[int] = None,
     ):
+        if isinstance(column_names, str):
+            column_names = [column_names]
+        else:
+            column_names = [v for v in column_names]  # convert to list from other types such as series
         ViewRepresentation.__init__(
             self, column_names=column_names, node_name="TableDescription", column_types=column_types,
         )
@@ -746,6 +758,10 @@ class TableDescription(ViewRepresentation):
         s = s + ")"
         return s
 
+    def example_values_to_sql_str_list(self, db_model) -> List[str]:
+        assert self.head is not None
+        return db_model.table_values_to_sql_str_list(self.head, result_name=self.table_name)
+
     def get_tables(self):
         """get a dictionary of all tables used in an operator DAG,
         raise an exception if the values are not consistent"""
@@ -762,7 +778,7 @@ class TableDescription(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.table_def_to_sql(
+        return db_model.table_def_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -776,11 +792,6 @@ class TableDescription(ViewRepresentation):
         return rep
 
     # comparable to other table descriptions
-    def __lt__(self, other):
-        if not isinstance(other, TableDescription):
-            return True
-        return self.key.__lt__(other.key)
-
     def __eq__(self, other):
         if not isinstance(other, TableDescription):
             return False
@@ -797,7 +808,7 @@ def describe_table(
     qualifiers=None,
     sql_meta=None,
     column_types=None,
-    row_limit: typing.Optional[int] = 7,
+    row_limit: Optional[int] = 7,
     keep_sample=True,
     keep_all=False,
     guess_column_types=True,
@@ -1122,7 +1133,7 @@ class ExtendNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.extend_to_sql(
+        return db_model.extend_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1266,7 +1277,7 @@ class ProjectNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.project_to_sql(
+        return db_model.project_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1336,7 +1347,7 @@ class SelectRowsNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.select_rows_to_sql(
+        return db_model.select_rows_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1410,7 +1421,7 @@ class SelectColumnsNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.select_columns_to_sql(
+        return db_model.select_columns_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1484,7 +1495,7 @@ class DropColumnsNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.drop_columns_to_sql(
+        return db_model.drop_columns_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1570,7 +1581,7 @@ class OrderRowsNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.order_to_sql(
+        return db_model.order_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1670,7 +1681,7 @@ class RenameColumnsNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.rename_to_sql(
+        return db_model.rename_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1787,7 +1798,7 @@ class NaturalJoinNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.natural_join_to_sql(
+        return db_model.natural_join_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -1906,7 +1917,7 @@ class ConcatRowsNode(ViewRepresentation):
     def to_near_sql_implementation(
         self, db_model, *, using, temp_id_source, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
-        return db_model.concat_rows_to_sql(
+        return db_model.concat_rows_to_near_sql(
             self,
             using=using,
             temp_id_source=temp_id_source,
@@ -2016,6 +2027,67 @@ class ConvertRecordsNode(ViewRepresentation):
         )
 
 
+class SQLNode(ViewRepresentation):
+    def __init__(self, *, sql: Union[str, List[str]], column_names: List[str], view_name: str):
+        if isinstance(sql, str):
+            sql = sql.splitlines(keepends=False)
+            sql = [v for v in sql if len(v.strip()) > 0]
+        assert isinstance(sql, list)
+        assert len(sql) > 0
+        assert all([isinstance(v, str) for v in sql])
+        assert isinstance(view_name, str)
+        self.sql = sql.copy()
+        self.view_name = view_name
+        ViewRepresentation.__init__(
+            self,
+            column_names=column_names,
+            node_name="SQLNode",
+        )
+
+    def apply_to(self, a, *, target_table_key=None):
+        return self
+
+    def _equiv_nodes(self, other):
+        if not isinstance(other, SQLNode):
+            return False
+        if self.view_name != other.view_name:
+            return False
+        if self.column_names != other.column_names:
+            return False
+        if self.sql != other.sql:
+            return False
+        return True
+
+    def columns_used_from_sources(self, using=None):
+        return []
+
+    def to_python_implementation(self, *, indent=0, strict=True, print_sources=True):
+        s = (
+                "SQLNode(sql=" + str(self.sql)
+                + ", column_names=" + str(self.column_names)
+                + ", view_name=" + self.view_name.__repr__()
+                + ")")
+        return s
+
+    def to_near_sql_implementation(
+        self, db_model, *, using, temp_id_source, sql_format_options=None
+    ) -> data_algebra.near_sql.NearSQL:
+        quoted_query_name = None
+        quoted_query_name = db_model.quote_identifier(self.view_name)
+        near_sql = data_algebra.near_sql.NearSQLRawQStep(
+            prefix=self.sql,
+            quoted_query_name=quoted_query_name,
+            sub_sql=None,
+            suffix=None,
+            annotation="user supplied SQL",
+            add_select=False,
+        )
+        return near_sql
+
+    def eval_implementation(self, *, data_map, data_model, narrow):
+        return data_map[self.view_name]
+
+
 def ex(d, *, data_model=None, narrow=True, allow_limited_tables=False):
     """
     Evaluate operators with respect to Pandas data frames already stored in the operator chain.
