@@ -1,18 +1,14 @@
-from types import SimpleNamespace
+"""
+Type defs for data operations.
+"""
+
+
 import collections
 
 import data_algebra.expr_rep
 import data_algebra.cdata
 
 
-class PipeStep:
-    def __init__(self):
-        pass
-
-    def apply_to(self, other, **kwargs):
-        raise NotImplementedError("base class called")
-
-
 class OperatorPlatform:
     """Abstract class representing ability to apply data_algebra operations."""
 
@@ -77,8 +73,6 @@ class OperatorPlatform:
         """
         if isinstance(other, OperatorPlatform):
             return self.apply_to(other)
-        if isinstance(other, PipeStep):
-            return other.apply_to(other)
         return self.act_on(other)
 
     def __rshift__(self, other):  # override self >> other
@@ -92,10 +86,20 @@ class OperatorPlatform:
         # can't use type >> type if only __rrshift__ is defined (must have __rshift__ in this case)
         if isinstance(other, OperatorPlatform):
             return other.apply_to(self)
-        if isinstance(other, PipeStep):
-            return other.apply_to(self)
         raise TypeError("unexpected type: " + str(type(other)))
 
+    # imitate a method
+    def use(self, user_function, *args, **kwargs):
+        """
+        Apply f as if it was a method on this chain.
+        Defined as return f(self, *args, **kwargs).
+
+        :param user_function: function to apply
+        :param *args: additional positional arguments
+        :param **kwargs: additional keyword arguments
+        """
+        return user_function(self, *args, **kwargs)
+
     # composition
     def add(self, other):
         """
@@ -179,8 +183,7 @@ class OperatorPlatform:
     def convert_records(self, record_map):
         raise NotImplementedError("base class called")
 
-    def map_records(
-        self, blocks_in=None, blocks_out=None):
+    def map_records(self, blocks_in=None, blocks_out=None):
         if (blocks_in is None) and (blocks_out is None):
             return self  # NO-OP, return source ops
         return self.convert_records(
